import urllib.request
import urllib.parse
import json
from weather_gov import endpoints

class Client:
    def __init__(self, email: str = None) -> None:
        self.API_BASE = "https://api.weather.gov"
        self.user_agent = f"python-weather-gov|{email}"
        self.alerts = endpoints.Alerts(self)
        self.gridpoints = endpoints.Gridpoints(self)
        self.stations = endpoints.Stations(self)
        self.offices = endpoints.Offices(self)
        self.points = endpoints.Points(self)
        self.radar = endpoints.Radar(self)
        self.products = endpoints.Products(self)
        self.zones = endpoints.Zones(self)

    def get(self, endpoint: str,
            params: dict = None,
            json_response: bool = True,
            feature_flags: list = None) -> dict:
    
        url = f"{self.API_BASE}/{endpoint}"
        if params: 
            url += "?" + urllib.parse.urlencode(params)
        
        headers = {'User-Agent': self.user_agent}
        
        if feature_flags:
            headers['Feature-Flags'] = feature_flags
        
        req = urllib.request.Request(url, data=None, headers=headers)
        with urllib.request.urlopen(req) as response: # ToDo: Handle urllib request errors
            if json_response:
                return json.loads(response.read().decode('utf-8'))
            return response.read().decode('utf-8')

    def glossary(self, **params) -> dict:
        return self.get("glossary", params)



""" Validation constants. Not completely implemented for custom parameters."""

VALID_WFO = ["AKQ", "ALY", "BGM", "BOX", "BTV", "BUF", "CAE",
            "CAR", "CHS", "CLE", "CTP", "GSP", "GYX", "ILM",
            "ILN", "LWX", "MHX", "OKX", "PBZ", "PHI", "RAH",
            "RLX", "RNK", "ABQ", "AMA", "BMX", "BRO", "CRP",
            "EPZ", "EWX", "FFC", "FWD", "HGX", "HUN", "JAN",
            "JAX", "KEY", "LCH", "LIX", "LUB", "LZK", "MAF",
            "MEG", "MFL", "MLB", "MOB", "MRX", "OHX", "OUN",
            "SHV", "SJT", "SJU", "TAE", "TBW", "TSA", "ABR",
            "APX", "ARX", "BIS", "BOU", "CYS", "DDC", "DLH",
            "DMX", "DTX", "DVN", "EAX", "FGF", "FSD", "GID",
            "GJT", "GLD", "GRB", "GRR", "ICT", "ILX", "IND",
            "IWX", "JKL", "LBF", "LMK", "LOT", "LSX", "MKX",
            "MPX", "MQT", "OAX", "PAH", "PUB", "RIW", "SGF",
            "TOP", "UNR", "BOI", "BYZ", "EKA", "FGZ", "GGW",
            "HNX", "LKN", "LOX", "MFR", "MSO", "MTR", "OTX",
            "PDT", "PIH", "PQR", "PSR", "REV", "SEW", "SGX",
            "SLC", "STO", "TFX", "TWC", "VEF", "AER", "AFC",
            "AFG", "AJK", "ALU", "GUM", "HPA", "HFO", "PPG",
            "STU", "NH1", "NH2", "ONA", "ONP"]

# States
VALID_STATE = ["AL", "AK", "AS", "AR", "AZ", "CA", "CO", "CT",
                   "DE", "DC", "FL", "GA", "GU", "HI", "ID", "IL",
                   "IN", "IA", "KS", "KY", "LA", "ME", "MD", "MA",
                   "MI", "MN", "MS", "MO", "MT", "NE", "NV", "NH",
                   "NJ", "NM", "NY", "NC", "ND", "OH", "OK", "OR",
                   "PA", "PR", "RI", "SC", "SD", "TN", "TX", "UT",
                   "VT", "VI", "VA", "WA", "WV", "WI", "WY"]

# Marine area codes (NWS Directive 10-302)
VALID_MARINE_AREA_CODE = ["AM", "AN", "GM", "LC", "LE", "LH", "LM",
                           "LO","LS", "PH", "PK", "PM", "PS", "PZ", "SL"]

VALID_MARINE_REGION_CODE = ["AL", "AT", "GL", "GM", "PA", "PI"]

VALID_LAND_REGION_CODE = ["AR", "CR", "ER", "PR", "SR", "WR"]

VALID_AREA_CODE = VALID_STATE + VALID_MARINE_AREA_CODE

VALID_REGION = ["AL", "AT", "GL", "GM", "PA", "PI"]

VALID_REGION_TYPE = ["land", "marine"]

VALID_URGENCY = ["Immediate", "Expected", "Future", "Past", "Unknown"]

VALID_SEVERITY = ["Extreme", "Severe", "Moderate", "Minor", "Unknown"]

VALID_CERTAINTY = ["Observed", "Likely", "Possible", "Unlikely", "Unknown"]

VALID_UNITS = ["us","si"]

VALID_GRIDPOINT_FORCAST_FEATURES = ["forecast_temperature_qv", "forecast_wind_speed_qv"]

VALID_ALERT_STATUS = ["Actual", "Exercise", "System", "Test", "Draft"]

VALID_ALERT_MESSAGE_TYPE = ["Alert", "Update", "Cancel", "Ack", "Error"]

VALID_ALERT_CATEGORY = ["Met", "Geo", "Safety", "Security", "Rescue", "Fire",
                        "Health", "Env", "Transport", "Infra", "CBRNE", "Other"]

VALID_ALERT_SEVERITY = VALID_SEVERITY

VALID_ALERT_CERTAINTY = VALID_CERTAINTY

VALID_ALERT_URGENCY = VALID_URGENCY

VALID_ALERT_RESPONSE = ["Shelter", "Evacuate", "Prepare", "Execute",
                        "Avoid", "Monitor", "Assess", "AllClear", "None"]

VALID_ZONE_TYPE = [ "land", "marine", "forecast", "public", "coastal",
                   "offshore", "fire", "county"]



from weather_gov import validations


class Base_Endpoint:
    def __init__(self, parent) -> None:
        self.parent = parent
        self._wfo = validations.VALID_WFO
        self._region = validations.VALID_REGION
        self._region_type = validations.VALID_REGION_TYPE
        self._urgency = validations.VALID_URGENCY
        self._severity = validations.VALID_SEVERITY
        self._certainty = validations.VALID_CERTAINTY
        self._zone_type = validations.VALID_ZONE_TYPE


class Alerts(Base_Endpoint):
    def __call__(self, alertID: str = None, **params) -> dict:

        if alertID:
            return self.parent.get(f"alerts/{alertID}", params)
        return self.parent.get("alerts", params)

    def active(self, **params) -> dict:

        return self.parent.get("alerts/active", params)
    
    def active_count(self, **params) -> dict:
        return self.parent.get("alerts/active/count", params)
    
    def active_zone(self, zoneID: str, **params) -> dict:
        return self.parent.get(f"alerts/active/zone/{zoneID}", params)

    def active_area(self, area: str, **params) -> dict:
        return self.parent.get(f"alerts/active/area/{area}", params)
    
    def active_region(self, region: str, **params) -> dict:
        
        return self.parent.get(f"alerts/active/region/{region}", params)
    
    def types(self, **params) -> dict:
        return self.parent.get("alerts/types", params)


class Gridpoints(Base_Endpoint):
    def __init__(self, parent) -> None:
        super().__init__(parent)
        self._features = validations.VALID_GRIDPOINT_FORCAST_FEATURES
    
    def __call__(self, wfo:str, x:int, y:int, **params) -> dict:
        if wfo not in self._wfo:
            raise ValueError(f"{wfo} is not a valid WFO")
        self.parent.get(f"gridpoints/{wfo}/{x}/{y}", params)
        
    def forcast(self, wfo:str, x:int, y:int, features: list = None, **params) -> dict:
        for feature in features:
            if feature not in self._features:
                raise ValueError(f"{feature} is not a valid feature")
        if wfo not in self._wfo:
            raise ValueError(f"{wfo} is not a valid WFO")
        return self.parent.get(f"gridpoints/{wfo}/{x}/{y}/forecast", params, feature_flags=features)
    
    def forecast_hourly(self, wfo:str, x:int, y:int, features: list = None, **params) -> dict:
        for feature in features:
            if feature not in self._features:
                raise ValueError(f"{feature} is not a valid feature")
        if wfo not in self._wfo:
            raise ValueError(f"{wfo} is not a valid WFO")
        return self.parent.get(f"gridpoints/{wfo}/{x}/{y}/forecast/hourly", params, feature_flags=features)

    def stations(self, wfo:str, x:int, y:int, **params) -> dict:
        if wfo not in self._wfo:
            raise ValueError(f"{wfo} is not a valid WFO")
        return self.parent.get(f"gridpoints/{wfo}/{x}/{y}/stations", params)


class Stations:
    def __init__(self, parent) -> None:
        self.parent = parent
        
    def __call__(self, **params) -> dict: 
        return self.parent.get("stations", params)
    
    def id(self, stationID: str, **params) -> dict:
        return self.parent.get(f"stations/{stationID}", params)
    
    def id_observations(self, stationID: str, **params) -> dict:
        return self.parent.get(f"stations/{stationID}/observations", params)
    
    def id_observations_latest(self, stationID: str, **params) -> dict:
        return self.parent.get(f"stations/{stationID}/observations/latest", params)
    
    def id_observations_time(self, stationID: str, time:str,  **params) -> dict:
        return self.parent.get(f"stations/{stationID}/observations/{time}", params)
    
class Offices(Base_Endpoint):
    def __init__(self, parent) -> None:
        super().__init__(parent)
        
    def __call__(self, **params) -> dict:
        return self.parent.get("offices", params)
    
    def id(self, officeID: str, **params) -> dict:
        if officeID not in self._wfo:
            raise ValueError(f"{officeID} is not a valid WFO")
        return self.parent.get(f"offices/{officeID}", params)
    
    def headlines(self, officeID: str, **params) -> dict:
        if officeID not in self._wfo:
            raise ValueError(f"{officeID} is not a valid WFO")
        return self.parent.get(f"offices/{officeID}/headlines", params)

    def headline_id(self, officeID: str, headlineID: str, **params) -> dict:
        if officeID not in self._wfo:
            raise ValueError(f"{officeID} is not a valid WFO")
        return self.parent.get(f"offices/{officeID}/headlines/{headlineID}", params)


class Points:
    def __init__(self, parent) -> None:
        self.parent = parent

    def __call__(self) -> dict:
        raise NotImplementedError("Points() method is not implemented")

    def id(self, pointID: str, **params) -> dict:
        return self.parent.get(f"points/{pointID}", params)
        
        
class Radar(Base_Endpoint):
    def __init__(self, parent) -> None:
        super().__init__(parent)
        
    def __call__(self) -> dict:
        raise NotImplementedError("Radar() method is not implemented")

    def servers(self, **params) -> dict:
        return self.parent.get("radar/servers", params)

    def server_id(self, serverID: str, **params) -> dict:
        return self.parent.get(f"radar/servers/{serverID}", params)

    def stations(self, **params) -> dict:
        return self.parent.get("radar/stations", params)

    def station_id(self, stationID: str, **params) -> dict:
        return self.parent.get(f"radar/stations/{stationID}", params)

    def station_id_alarms(self, stationID: str, **params) -> dict:
        return self.parent.get(f"radar/stations/{stationID}/alarms", params)

    def queues(self, host:str, **params) -> dict:
        return self.parent.get(f"radar/queues/{host}", params)


class Products:
    def __init__(self, parent) -> None:
        self.parent = parent

    def __call__(self, **params) -> dict:
        return self.parent.get("products", params)

    def locations(self, **params) -> dict:
        return self.parent.get("products/locations", params)

    def types(self, **params) -> dict:
        return self.parent.get("products/types", params)

    def id(self, productID: str, **params) -> dict:
        return self.parent.get(f"products/{productID}", params)

    def type_id(self, typeID: str, **params) -> dict:
        return self.parent.get(f"products/types/{typeID}", params)

    def type_id_locations(self, typeID: str, **params) -> dict:
        return self.parent.get(f"products/types/{typeID}/locations", params)

    def location_id_types(self, locationID: str, **params) -> dict:
        return self.parent.get(f"products/locations/{locationID}/types", params)

    def type_id_location_id(self, typeID: str, locationID: str, **params) -> dict:
        return self.parent.get(f"products/types/{typeID}/locations/{locationID}", params)


class Zones(Base_Endpoint):
    def __init__(self, parent) -> None:
        super().__init__(parent)

    def __call__(self, **params) -> dict:
        self.parent.get("zones", params)

    def type(self, zoneType: str, **params) -> dict:
        if zoneType not in self._zone_type:
            raise ValueError(f"{zoneType} is not a valid zone type")
        return self.parent.get(f"zones/types/{zoneType}", params)

    def type_zone_id(self, zoneType: str, zoneID: str, **params) -> dict:
        if zoneType not in self._zone_type:
            raise ValueError(f"{zoneType} is not a valid zone type")
        return self.parent.get(f"zones/types/{zoneType}/{zoneID}", params)

    def type_zone_id_forecast(self, zoneType: str, zoneID: str, **params) -> dict:
        if zoneType not in self._zone_type:
            raise ValueError(f"{zoneType} is not a valid zone type")
        return self.parent.get(f"zones/types/{zoneType}/{zoneID}/forecast", params)

    def forecast_id_observations(self, zoneID: str, **params) -> dict:
        return self.parent.get(f"zones/forecasts/{zoneID}/observations", params)

    def forecast_id_stations(self, zoneID: str, **params) -> dict:
        return self.parent.get(f"zones/forecasts/{zoneID}/stations", params)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()
    
setuptools.setup(
    name="weather_gov",
    version="0.2",
    #scripts=["weather_gov"],
    author="TJ Courier",
    author_email="pycepticus@gmail.com",
    description="A Python wrapper for the National Weather Service's Weather.gov API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/spectrshiv/python-weather_gov",
    packages=setuptools.find_namespace_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Atmospheric Science",
    ],
    py_modules=["urllib", "json"],
    
)